% ***********************************************************************
% Copyright (c) Laura Sevilla-Lara and Erik G. Learned-Miller, 2012.
% ***********************************************************************

function positions= trackDF(params)

%%% INPUT PARAMETERS FOR IMAGE SEQUENCE 

% -- file_path: path where frame sequence is found, including the invariant
% part of the frames name 
% -- file_format: string containing the format of the frames 
% -- output_name: name of the output file with the results of tracking 
% -- start_fr: number of the first frame where the target appears 
% -- end_fr: number of the last frame where you want to track the target
% -- init_pos: position of the target in the first frame, specified as [row, col]
% -- wsize: size of target specified as [height, width]

%%% INPUT PARAMETERS FOR DISTRIBUTION FIELDS 

% -- nbins: number of bins used to quantize the feature space in a DF  
% -- feat_width: width of kernel for convolving a DF in feature space 
% -- feat_sig: std dev of the gaussian kernel for convolving a DF in feat space
% -- sp_width: set of widths of the kernels for convolving a DF in image space
% -- sp_sig: set of std devs of the gaussian kernels for convolving a DF in image space
% -- max_shift: maximum number of pixels the target is expected to move 

%%% OUTPUT

% -- positions: n-by-2 array containing the positions of the object at each
% frame 


% rename variables for clarity
start_fr = params.start_fr;
end_fr = params.end_fr;
init_pos = params.init_pos;
wsize = params.wsize;
nbins = params.nbins;
feat_width = params.feat_width; 
feat_sig = params.feat_sig;
sp_width = params.sp_width;
sp_sig = params.sp_sig;
max_shift = params.max_shift;
depth=50;
initSize=wsize(1);
newdepth=depth;
% Read first frame and crop target
f1 = double(imread(sprintf('%s%03d.%s', params.file_path, start_fr, params.file_format)));
f1 = f1(init_pos(1):init_pos(1)+wsize(1)-1, init_pos(2):init_pos(2)+wsize(2)-1);
%show the object of interest to be tracked
imshow(uint8(f1)); pause(0.3);
% Compute and smooth DF of target with different levels of blur
df = img2df(f1, nbins);
for i=1:length(sp_width)
    target_models{i} = smoothDF(df, [sp_width(i) feat_width], [sp_sig(i), feat_sig]);
end;

% create the name of the files to save the results 
pos_file = sprintf('%s.mat', params.output_name);

% track the object along the sequence 
num_frames = end_fr - start_fr; 
positions = zeros(num_frames,4);%stores the position coordinates of top-left
%corner and height-width of bounding box
last_motion = [0, 0];

fprintf('Processing frame number \n');
scales=[1 .95 1.05];%list the scales at which you want to scale the target model
init_posScale=ones(size(scales,2),2);
crop_params=ones(size(scales,2),4);
    for i=1:size(scales,2)
        init_posScale(i,1)=init_pos(1)+round(wsize(1)*(1-scales(i))/2);
        init_posScale(i,2)=init_pos(2)+round(wsize(2)*(1-scales(i))/2);
    end
    
for i=1:num_frames

    % read image
    f2 = double(imread(sprintf('%s%03d.%s', params.file_path, i+start_fr, params.file_format)));
    
    for s=1:size(scales,2)
        init_posScale(s,1)=init_pos(1)+round(wsize(1)*(1-scales(s))/2);
        init_posScale(s,2)=init_pos(2)+round(wsize(2)*(1-scales(s))/2);
    % compute new starting position using last motion
    init_posScale(s,:)= computeStartingPoint(init_posScale(s,:), last_motion,round( wsize*scales(s)), size(f2));
    
    % crop the image around the starting position for speed
    crop_params(s,:)= computeCropParams(init_posScale(s,:), size(f2), round(wsize*scales(s)), max_shift, sp_width(end));
    end
    %f2 = f2(crop_params(1):crop_params(1)+crop_params(3)-1, crop_params(2):crop_params(2)+crop_params(4)-1);
    
    % find target in frame 
    %crop_params
    [pts INDX]= findTargetHier(target_models, f2, init_posScale-crop_params(:,1:2)+1, wsize, sp_width, sp_sig, feat_width, feat_sig, nbins,scales,crop_params); 
    end_pos = pts(end, :);
    
    %rescaling the target DF and rectangle width
    wsize=round(wsize*scales(INDX));
    init_pos=init_posScale(INDX,:);
    newdepth=depth*initSize/wsize(1)
    for p=1:size(target_models,2)
%         if(scales(INDX)==1)
%             continue;
%         end
        for k=1:size(target_models{1},3)
           out_models{p}(:,:,k)=imresize(target_models{p}(:,:,k),scales(INDX));
           %out_models{p}(:,:,k)=re_target_models{p,indx}(:,:,k);
        end     
    end
    clear target_models;
    for p=1:size(out_models,2)
        for k=1:size(out_models{1},3)
           target_models{p}(:,:,k)=out_models{p}(:,:,k);
           %out_models{p}(:,:,k)=re_target_models{p,indx}(:,:,k);
        end     
    end
    clear out_models;
    % update target model 
   % f2 = f2(end_pos(1):end_pos(1)+ round(wsize(1)*scales(INDX))-1, end_pos(2):end_pos(2)+round(wsize(2)*scales(INDX)-1));
  im = uint8(drawRect(f2,init_posScale(INDX,:) , wsize, 'r', 2));
 %imwrite(im,sprintf('./out/%s%03d.jpg', 'clock_trackM',i));
% Text = sprintf('new depth(inches) is %s',num2str(newdepth));
% H = vision.TextInserter(Text);
% H.Color = [1.0 0 0];
% H.FontSize = 8;
% H.Location = [size(im,1)/2 10];
%  im=step(H, im);
  imshow(im);
    pause(0.01);
   f2=f2(init_posScale(INDX,1):init_posScale(INDX,1)+size(target_models{1},1)-1, init_posScale(INDX,2):init_posScale(INDX,2)+size(target_models{2},2)-1);
   df2 = img2df(f2, nbins);
    for j = 1:length(sp_width)
        df2_s = smoothDF(df2, [sp_width(j) feat_width], [sp_sig(j), feat_sig]);
        %size(df2_s)
        %size(target_models{j})
        target_models{j} = 0.95.*target_models{j} + 0.05.*df2_s;
    end;
    positions(i,[3 4])=wsize;
    % store result
    end_pos = end_pos+crop_params(INDX,1:2)-1;
    positions(i, [1 2]) = end_pos;
    fprintf(1,' %d ', i);
    if mod(i, 10) == 0
        fprintf('\n');
    end;
    last_motion = end_pos - init_pos;
    %init_pos = end_pos; 

    save(pos_file, 'positions');

end;


